@@ -2,33 +2,36 @@
 Keyring implementation support
 """
 
-from __future__ import absolute_import
-
+import os
 import abc
 import logging
-import importlib
+import operator
+
+from typing import Optional
 
 try:
-    import pkg_resources
+    from importlib import metadata  # type: ignore
 except ImportError:
-    pass
+    import importlib_metadata as metadata  # type: ignore
 
-from . import errors, util
-from . import backends
+from . import credentials, errors, util
 from .util import properties
-from .py27compat import add_metaclass, filter
-
 
 log = logging.getLogger(__name__)
 
 
+by_priority = operator.attrgetter('priority')
+_limit = None
+
+
 class KeyringBackendMeta(abc.ABCMeta):
     """
     A metaclass that's both an ABCMeta and a type that keeps a registry of
     all (non-abstract) types.
     """
+
     def __init__(cls, name, bases, dict):
-        super(KeyringBackendMeta, cls).__init__(name, bases, dict)
+        super().__init__(name, bases, dict)
         if not hasattr(cls, '_classes'):
             cls._classes = set()
         classes = cls._classes
@@ -36,13 +39,12 @@ class KeyringBackendMeta(abc.ABCMeta):
             classes.add(cls)
 
 
-@add_metaclass(KeyringBackendMeta)
-class KeyringBackend(object):
+class KeyringBackend(metaclass=KeyringBackendMeta):
     """The abstract base class of the keyring, every backend must implement
     this interface.
     """
 
-    #@abc.abstractproperty
+    # @abc.abstractproperty
     def priority(cls):
         """
         Each backend class must supply a priority, a number (float or integer)
@@ -64,47 +66,107 @@ class KeyringBackend(object):
     def viable(cls):
         with errors.ExceptionRaisedContext() as exc:
             cls.priority
-        return not bool(exc)
+        return not exc
+
+    @classmethod
+    def get_viable_backends(cls):
+        """
+        Return all subclasses deemed viable.
+        """
+        return filter(operator.attrgetter('viable'), cls._classes)
+
+    @properties.ClassProperty
+    @classmethod
+    def name(cls):
+        """
+        The keyring name, suitable for display.
+
+        The name is derived from module and class name.
+        """
+        parent, sep, mod_name = cls.__module__.rpartition('.')
+        mod_name = mod_name.replace('_', ' ')
+        return ' '.join([mod_name, cls.__name__])
+
+    def __str__(self):
+        keyring_class = type(self)
+        return "{}.{} (priority: {:g})".format(
+            keyring_class.__module__, keyring_class.__name__, keyring_class.priority
+        )
 
     @abc.abstractmethod
-    def get_password(self, service, username):
-        """Get password of the username for the service
-        """
+    def get_password(self, service, username) -> Optional[str]:
+        """Get password of the username for the service"""
         return None
 
     @abc.abstractmethod
     def set_password(self, service, username, password):
-        """Set password for the username of the service
+        """Set password for the username of the service.
+
+        If the backend cannot store passwords, raise
+        PasswordSetError.
         """
         raise errors.PasswordSetError("reason")
 
     # for backward-compatibility, don't require a backend to implement
     #  delete_password
-    #@abc.abstractmethod
+    # @abc.abstractmethod
     def delete_password(self, service, username):
         """Delete the password for the username of the service.
+
+        If the backend cannot delete passwords, raise
+        PasswordDeleteError.
         """
         raise errors.PasswordDeleteError("reason")
 
-class Crypter(object):
-    """Base class providing encryption and decryption
-    """
+    # for backward-compatibility, don't require a backend to implement
+    #  get_credential
+    # @abc.abstractmethod
+    def get_credential(
+        self, service, username
+    ) -> Optional[credentials.SimpleCredential]:
+        """Gets the username and password for the service.
+        Returns a Credential instance.
+
+        The *username* argument is optional and may be omitted by
+        the caller or ignored by the backend. Callers must use the
+        returned username.
+        """
+        # The default implementation requires a username here.
+        if username is not None:
+            password = self.get_password(service, username)
+            if password is not None:
+                return credentials.SimpleCredential(username, password)
+        return None
+
+    def set_properties_from_env(self):
+        """For all KEYRING_PROPERTY_* env var, set that property."""
+
+        def parse(item):
+            key, value = item
+            pre, sep, name = key.partition('KEYRING_PROPERTY_')
+            return sep and (name.lower(), value)
+
+        props = filter(None, map(parse, os.environ.items()))
+        for name, value in props:
+            setattr(self, name, value)
+
+
+class Crypter:
+    """Base class providing encryption and decryption"""
 
     @abc.abstractmethod
     def encrypt(self, value):
-        """Encrypt the value.
-        """
+        """Encrypt the value."""
         pass
 
     @abc.abstractmethod
     def decrypt(self, value):
-        """Decrypt the value.
-        """
+        """Decrypt the value."""
         pass
 
+
 class NullCrypter(Crypter):
-    """A crypter that does nothing
-    """
+    """A crypter that does nothing"""
 
     def encrypt(self, value):
         return value
@@ -113,21 +175,6 @@ class NullCrypter(Crypter):
         return value
 
 
-def _load_backend(name):
-    "Load a backend by name"
-    package = backends.__package__ or backends.__name__
-    mod = importlib.import_module('.'+name, package)
-    # invoke __name__ on each module to ensure it's loaded in demand-import
-    # environments
-    mod.__name__
-
-def _load_backends():
-    "ensure that all keyring backends are loaded"
-    backends = ('file', 'Gnome', 'Google', 'keyczar', 'kwallet', 'multi',
-        'OS_X', 'pyfs', 'SecretService', 'Windows')
-    list(map(_load_backend, backends))
-    _load_plugins()
-
 def _load_plugins():
     """
     Locate all setuptools entry points by the name 'keyring backends'
@@ -146,18 +193,16 @@ def _load_plugins():
 
     `initialize_func` is optional, but will be invoked if callable.
     """
-    if 'pkg_resources' not in globals():
-        return
-    group = 'keyring.backends'
-    entry_points = pkg_resources.iter_entry_points(group=group)
+    entry_points = metadata.entry_points().get('keyring.backends', [])
     for ep in entry_points:
         try:
-            log.info('Loading %s', ep.name)
+            log.debug('Loading %s', ep.name)
             init_func = ep.load()
             if callable(init_func):
                 init_func()
         except Exception:
-            log.exception("Error initializing plugin %s." % ep)
+            log.exception(f"Error initializing plugin {ep}.")
+
 
 @util.once
 def get_all_keyring():
@@ -165,16 +210,7 @@ def get_all_keyring():
     Return a list of all implemented keyrings that can be constructed without
     parameters.
     """
-    _load_backends()
-
-    def is_class_viable(keyring_cls):
-        try:
-            keyring_cls.priority
-        except RuntimeError:
-            return False
-        return True
-
-    all_classes = KeyringBackend._classes
-    viable_classes = filter(is_class_viable, all_classes)
-    return list(util.suppress_exceptions(viable_classes,
-        exceptions=TypeError))
+    _load_plugins()
+    viable_classes = KeyringBackend.get_viable_backends()
+    rings = util.suppress_exceptions(viable_classes, exceptions=TypeError)
+    return list(rings)
