@@ -2,13 +2,10 @@
 Core API functions and initialization routines.
 """
 
+import configparser
 import os
 import sys
 import logging
-import operator
-
-from .py27compat import configparser, filter
-from .py33compat import max
 
 from . import backend
 from .util import platform_ as platform
@@ -21,53 +18,88 @@ _keyring_backend = None
 
 
 def set_keyring(keyring):
-    """Set current keyring backend.
-    """
+    """Set current keyring backend."""
     global _keyring_backend
     if not isinstance(keyring, backend.KeyringBackend):
         raise TypeError("The keyring must be a subclass of KeyringBackend")
     _keyring_backend = keyring
+    keyring.set_properties_from_env()
 
 
 def get_keyring():
-    """Get current keyring backend.
-    """
+    """Get current keyring backend."""
+    if _keyring_backend is None:
+        init_backend()
     return _keyring_backend
 
 
-def get_password(service_name, username):
-    """Get password from the specified service.
+def disable():
     """
-    return _keyring_backend.get_password(service_name, username)
+    Configure the null keyring as the default.
+    """
+    root = platform.config_root()
+    try:
+        os.makedirs(root)
+    except OSError:
+        pass
+    filename = os.path.join(root, 'keyringrc.cfg')
+    if os.path.exists(filename):
+        msg = f"Refusing to overwrite {filename}"
+        raise RuntimeError(msg)
+    with open(filename, 'w') as file:
+        file.write('[backend]\ndefault-keyring=keyring.backends.null.Keyring')
+
+
+def get_password(service_name, username):
+    """Get password from the specified service."""
+    return get_keyring().get_password(service_name, username)
 
 
 def set_password(service_name, username, password):
-    """Set password for the user in the specified service.
-    """
-    _keyring_backend.set_password(service_name, username, password)
+    """Set password for the user in the specified service."""
+    get_keyring().set_password(service_name, username, password)
 
 
 def delete_password(service_name, username):
-    """Delete the password for the user in the specified service.
-    """
-    _keyring_backend.delete_password(service_name, username)
+    """Delete the password for the user in the specified service."""
+    get_keyring().delete_password(service_name, username)
 
 
-def recommended(backend): return backend.priority >= 1
+def get_credential(service_name, username):
+    """Get a Credential for the specified service."""
+    return get_keyring().get_credential(service_name, username)
 
 
-by_priority = operator.attrgetter('priority')
+def recommended(backend):
+    return backend.priority >= 1
 
 
 def init_backend(limit=None):
     """
-    Load a keyring specified in the config file or infer the best available.
+    Load a detected backend.
     """
-    keyrings = filter(limit, backend.get_all_keyring())
+    set_keyring(_detect_backend(limit))
 
-    set_keyring(
-        load_config()
-        or max(keyrings, default=fail.Keyring, key=by_priority)
+
+def _detect_backend(limit=None):
+    """
+    Return a keyring specified in the config file or infer the best available.
+
+    Limit, if supplied, should be a callable taking a backend and returning
+    True if that backend should be included for consideration.
+    """
+
+    # save the limit for the chainer to honor
+    backend._limit = limit
+    return (
+        load_env()
+        or load_config()
+        or max(
+            # all keyrings passing the limit filter
+            filter(limit, backend.get_all_keyring()),
+            default=fail.Keyring(),
+            key=backend.by_priority,
+        )
     )
 
 
@@ -79,19 +111,12 @@ def _load_keyring_class(keyring_name):
 
     >>> popular_names = [
     ...      'keyring.backends.Windows.WinVaultKeyring',
-    ...      'keyring.backends.OS_X.Keyring',
+    ...      'keyring.backends.macOS.Keyring',
     ...      'keyring.backends.kwallet.DBusKeyring',
     ...      'keyring.backends.SecretService.Keyring',
     ...  ]
     >>> list(map(_load_keyring_class, popular_names))
     [...]
-
-    These legacy names are retained for compatibility.
-
-    >>> legacy_names = [
-    ...  ]
-    >>> list(map(_load_keyring_class, legacy_names))
-    [...]
     """
     module_name, sep, class_name = keyring_name.rpartition('.')
     __import__(module_name)
@@ -110,6 +135,14 @@ def load_keyring(keyring_name):
     return class_()
 
 
+def load_env():
+    """Load a keyring configured in the environment variable."""
+    try:
+        return load_keyring(os.environ['PYTHON_KEYRING_BACKEND'])
+    except KeyError:
+        pass
+
+
 def load_config():
     """Load a keyring using the config file in the config root."""
 
@@ -133,8 +166,10 @@ def load_config():
 
     except (configparser.NoOptionError, ImportError):
         logger = logging.getLogger('keyring')
-        logger.warning("Keyring config file contains incorrect values.\n" +
-                       "Config file: %s" % keyring_cfg)
+        logger.warning(
+            "Keyring config file contains incorrect values.\n"
+            + "Config file: %s" % keyring_cfg
+        )
         return
 
     return load_keyring(keyring_name)
@@ -147,7 +182,3 @@ def _load_keyring_path(config):
         sys.path.insert(0, path)
     except (configparser.NoOptionError, configparser.NoSectionError):
         pass
-
-
-# init the _keyring_backend
-init_backend()
